/******************************************************************************
* FILENAME:      LPC2000_CAN_SYS.C                                            *  
*                                                                             *
* VERSION:       V1.0                                                         *
*                                                                             *
* DESCRIPTION:   Philips LPC2000 CAN Controller Driver with SJA1000           *                     
*				 Demo by UART0 with KEIL MCB2100 Evaluation Board			  *
*				 Code runing indicate by LED4 with Timer1 interrupt			  *
*                UART0 tested with Receive "a" NOT LED1, Receive "b" NOT LED2 *
*                                                                             *
* TOOLS:         KEIL uVision3                                     			  *
*                ARM Development Suite ADS1.2								  *
*                KEIL MCB2100 Evaluation Board               				  *
*                                                                             *
* REVISION HISTORY                                                            *
* Version  Author           Date        Remarks								  *
* 1.0      Anthony Xie      10/10/2005    - -                                 *     
*******************************************************************************/

// -----------------------------------------------
// Project include files                                                     
// -----------------------------------------------
#include  "LPC2294.h"
#include  "config.h"
#include  ".\CAN\LPC2000_CAN.h"

//-----------------------------------------------
//Constant Definitions
//-----------------------------------------------
#define	  LED4CON	(1<<23)		// P1.23 control LED4
#define   LED3CON	(1<<22)		// P1.22 control LED3
#define	  LED2CON	(1<<21)		// P1.21 control LED2
#define   LED1CON	(1<<20)		// P1.20 control LED1


//---------------------------------------------
//Global Variable Definitions
//---------------------------------------------

	  //System Global Variable
UInt32 System_Counter,LED_Counter=0;
UInt8 LED_Flag=0;
UInt8 System_Status=0;

//---------------------------------------------
//* Functions
//---------------------------------------------
void init(void);
void delay(UInt32);
void __irq IRQ_Timer1(void);
void uart0_config(UInt32,UART0MODE);
void __irq IRQ_UART0(void);
void __irq IRQ_CAN1Tx(void);
void __irq IRQ_CAN2Rx(void);


//***************************************************************************
//* main()
//* Input(s): none.
//* Returns : none.
//* Description : main loop
//***************************************************************************
int  main(void)
{ 
  UInt32 i=0;
  plpc2000CANdriver_TXObj_t pCAN_Send_Data;
  plpc2000CANdriver_RXObj_t pCAN_Rcv_Data;
    
  delay(20);
  init();  

  Print_Chars_to_Screen( Demo_Start );

  for(;;)	
  {        

    if( LED_Flag == 0 ) IO1SET = LED4CON;	//Control LED4
    else IO1CLR = LED4CON;
    if( LED_Flag == 10) LED_Flag = 5;		// Solve ADS12 compiler problem, this condition will 
    										// never be met, and the two value can't be same

    switch( System_Status )
	  {
	    case 0:	    // wait input (1,2 choice) status
		  break;
		case 1:									   // "1" be pressed
		  lpc2000CANdriver_SetACFMode( LPC2000_ACC_BYPASS );
		  Print_Chars_to_Screen( Demo_Sent_Data );
		  Print_Chars_to_Screen( Data_Sent_Start_or_Not ); 
		  System_Status = 3;
		  break;
		case 2:									   // "2" be pressed
          lpc2000CANdriver_SetACFMode( LPC2000_ACC_ON );
		  Print_Chars_to_Screen( Demo_Sent_Data );
		  Print_Chars_to_Screen( Acceptance_Filter_Tables );
		  Print_Chars_to_Screen( Data_Sent_Start_or_Not ); 
		  System_Status = 3;
		  break;
		case 3:	   // wait input (y,n choice) status
		  break;
		case 5:	   // "y or Y" be pressed, 
		  pCAN_Send_Data = CAN_Send_Data;
          for( i=0;i<5;i++)
	        {
              Return_Value = lpc2000CANdriver_CertainTxBufTransmitMessage(LPC2000_CANCHANNEL_1,pCAN_Send_Data,LPC2000_TXB_1);

			  pCAN_Send_Data++;
		    }

		  System_Status = 0;
		  break;								
		case 7:	 // "N or n " be pressed
		  Print_Chars_to_Screen( Demo_Sent_Choice );
		  System_Status = 0;
		  break;
		default:
		  break;
	  }

    if( CAN_Rcv_Status == 1 ) //communication successed
	  {
          Print_Chars_to_Screen( Rcv_Data_OK );          
		  pCAN_Rcv_Data = CAN_Rcv_Data;	  // print to uart0 all data in Rcv_Buffer
		  for( i=0;i<CAN_Rcv_Data_Counter;i++)				  
		    {
			  Rcv_Data_Output_to_Screen( pCAN_Rcv_Data );
			  pCAN_Rcv_Data++; 
			}
		  CAN_Rcv_Data_Counter = 0;     // reset Rcv_Buffer, point to head, means empty it.
		  Print_Chars_to_Screen( Demo_Sent_Choice );

		  CAN_Rcv_Status = 0;
	  }


    if( CAN2Rx_Return_Message_Flag[0] != 0 ) // if one CAN2Rx interrupt happened
	  {										 // print massage of communication status
		
		for( i=1;i<10;i++)
		{
		  if( CAN2Rx_Return_Message_Flag[i] != 0)
		    {
              switch(i)
			    {
				  case 1: 
				    Print_Chars_to_Screen( CAN2Rx_Return_Message_EI );
				    Print_Chars_to_Screen( CAN_ERROR_STOP_DEMO );      
				    break;
 				  case 2: 
				    Print_Chars_to_Screen( CAN2Rx_Return_Message_DOI );
				    Print_Chars_to_Screen( CAN_ERROR_STOP_DEMO );
					break;
				  case 3: 
				    Print_Chars_to_Screen( CAN2Rx_Return_Message_WUI );
				    Print_Chars_to_Screen( CAN_ERROR_STOP_DEMO );
					break;
				  case 4: 
				    Print_Chars_to_Screen( CAN2Rx_Return_Message_EPI );
				    Print_Chars_to_Screen( CAN_ERROR_STOP_DEMO );
					break;
				  case 5: 
				    Print_Chars_to_Screen( CAN2Rx_Return_Message_ALI );
				    Print_Chars_to_Screen( CAN_ERROR_STOP_DEMO );
					break;
				  case 6: 
				    Print_Chars_to_Screen( CAN2Rx_Return_Message_BEI );
				    Print_Chars_to_Screen( CAN_ERROR_STOP_DEMO );
					break;
				  default:
				    break;
				}
			  CAN2Rx_Return_Message_Flag[i]=0;			  
			}
		}

	    CAN2Rx_Return_Message_Flag[0] = 0;
	  }

    if( (Return_Value != 0) && (Return_Value != 5))	  // Report CAN error status
	  {
	    switch(Return_Value)
		  {
		    case 10:
			  Print_Chars_to_Screen( CAN_Error_10 );
			  Print_Chars_to_Screen( CAN_ERROR_STOP_DEMO );
			  break;
		    case 30:
			  Print_Chars_to_Screen( CAN_Error_30 );
			  Print_Chars_to_Screen( CAN_ERROR_STOP_DEMO );
			  break;
		    case 31:
			  Print_Chars_to_Screen( CAN_Error_31 );
			  Print_Chars_to_Screen( CAN_ERROR_STOP_DEMO );
			  break;
		    case 40:
			  Print_Chars_to_Screen( CAN_Error_40 );
			  Print_Chars_to_Screen( CAN_ERROR_STOP_DEMO );
			  break;
		    case 50:
			  Print_Chars_to_Screen( CAN_Error_50 );
			  Print_Chars_to_Screen( CAN_ERROR_STOP_DEMO );
			  break;
		    case 60:
			  Print_Chars_to_Screen( CAN_Error_60 );
			  Print_Chars_to_Screen( CAN_ERROR_STOP_DEMO );
			  break;
		    case 90:
			  Print_Chars_to_Screen( CAN_Error_90 );
			  Print_Chars_to_Screen( CAN_ERROR_STOP_DEMO );
			  break;
		    case 100:
			  Print_Chars_to_Screen( CAN_Error_100 );
			  Print_Chars_to_Screen( CAN_ERROR_STOP_DEMO );
			  break;
		    case 110:
			  Print_Chars_to_Screen( CAN_Error_110 );
			  Print_Chars_to_Screen( CAN_ERROR_STOP_DEMO );
			  break;
		    case 120:
			  Print_Chars_to_Screen( CAN_Error_120 );
			  Print_Chars_to_Screen( CAN_ERROR_STOP_DEMO );
			  break;
		    case 130:
			  Print_Chars_to_Screen( CAN_Error_130 );
			  Print_Chars_to_Screen( CAN_ERROR_STOP_DEMO );
			  break;
			default:
			  break;
		  }

		Return_Value = 0;
	  }
      
  }
   
  return(0);
}

                               
//***************************************************************************
//* init()
//* Input(s): none.
//* Returns : none.
//* Description : initialization each function
//***************************************************************************
void init(void)
{
  UART0MODE Uart0_Config;


//-----------------------------------------------
//Config GPIO
//-----------------------------------------------
  PINSEL0  = 0x00000000;		// All be GPIO, Note: 2 bits config 1 pin!
  PINSEL0 |= 0x00000005;		// P0.0-TxD0,P0.1-RxD0

  PINSEL1 |= 0x00000000;		// All be GPIO, Note: 2 bits config 1 pin!
  PINSEL1 |= 0x00014000;		// P0.23-RD2, P0.24-TD2
//  PINSEL1 |= 0x00004000;		// P0.23-RD2, only enable receive for CAN2
  PINSEL1 |= 0x00040000;        // P0.25-RD1, Note:needn't for Self-Testing, Even not connect RD1			   
 
   
								// P1.16-23, has been configed to GPIO in <startup.s>
  IO1DIR = 0x00FF0000;          // P1.16 - 23 output, other input, 
                                // 0000 0000 1111 1111 0000 0000 0000 0000
  IO1CLR = 0x00FF0000;          // Turn off LED1 - LED8


//-----------------------------------------------
//Config Timer1
//Fcclk = Fosc*4 = 12MHz*4 = 48MHz
//Fpclk = Fcclk/2 = 48MHz/2 = 24MHz
//-----------------------------------------------  
  T1PR = 0;				// Disable prescaler
  T1MCR = 0x03;			// Using T1MR0 match T1TC,Enable interrupt,reset T1TC
  T1MR0 = Fpclk/100;	// 10ms each Timer interrupt
  T1TCR = 0x02;			// Reset T1TC, by set TCR:2
  T1TCR = 0x01;			// Must CLR TCR:2, then can Start T1TC


//-----------------------------------------------
//Config UART0
//-----------------------------------------------  
  Uart0_Config.Datalength = 8;
  Uart0_Config.Stopbit = 1;
  Uart0_Config.Paritybit = 0;
  uart0_config(9600,Uart0_Config);
  
  U0IER = 0x01;		// Enable Rx interrupt

//-----------------------------------------------
//Config CAN1,CAN2
//-----------------------------------------------  
  lpc2000CANdriver_CANInit( LPC2000_CANCHANNEL_1, LPC2000_CANDRIVER_CANBITRATE100K24MHZ );
  lpc2000CANdriver_CANInit( LPC2000_CANCHANNEL_2, LPC2000_CANDRIVER_CANBITRATE100K24MHZ );

  lpc2000CANdriver_SetACFMode( LPC2000_ACC_BYPASS );
  lpc2000CANdriver_LoadAcceptanceFilter ();

  CAN2IER = 0x000007FF;  //Enable all CAN2 interrupt


//-----------------------------------------------
//Config VIC, 
//In <Starup.s> enable IRQ by CLR I bit in CPSR
//Vectored interrupt Timer1 by slot-0
//Vectored interrupt UART0 by slot-1
//Vectored interrupt CAN2Rx by slot-2
//-----------------------------------------------  
  
  VICIntSelect = 0x00000000;			// All interrupt be IRQ
  VICVectCntl0 = 0x20|5;				// Assign Timer1 to IRQ Slot0
  VICVectAddr0 = (UInt32)IRQ_Timer1; 	// Assign IRQ Address for IRQ_Timer1
  VICVectCntl1 = 0x20|6;				// Assign UART0 to IRQ slot1
  VICVectAddr1 = (UInt32)IRQ_UART0; 	// Assign IRQ Address for IRQ_UART0  
  VICVectCntl2 = 0x20|27;				// Assign CAN2Rx to IRQ Slot2
  VICVectAddr2 = (UInt32)IRQ_CAN2Rx; 	// Assign IRQ Address for IRQ_CAN2Rx
  
  VICIntEnable = (1<<5)|(1<<6)|(1<<27);	// Enable Timer1+UART0+CAN2Rx interrupt

}

/****************************************************************************
* Name: Delay()
* Function: Delay 
* Input(s): delaytime		30 about 0.5s
* Returns : none
****************************************************************************/
void  delay(UInt32  delaytime)
{  UInt32  i;

   for(; delaytime>0; delaytime--) 
      for(i=0; i<50000; i++);
}

/****************************************************************************
* Name: uart0_config(UInt32,UART0MODE)
* Function: UART0 config 
* Input(s): Baudrate
*           U0_config
* Returns : none
****************************************************************************/
void uart0_config(UInt32 Baudrate,UART0MODE U0_config)
{
  UInt32 temp_uart0;
  
//-------------------------
// UART0 Baudrate config
//-------------------------
  U0LCR = 0x80;			// DLAB=1, Can use U0DLM, U0DLL 
  temp_uart0 = (Fpclk/16)/Baudrate;
  
  U0DLM = temp_uart0>>8;
  U0DLL = temp_uart0&0xFF;
  
//-------------------------
// UART0 mode config
//------------------------- 
  temp_uart0 = U0_config.Datalength - 5;				// Data Length prepare
  
  if( U0_config.Stopbit == 2 ) temp_uart0 |= 0x04;		// Data Stop bit prepare
  
  if( U0_config.Paritybit != 0 )						// Data Parity bit prepare
    {
      U0_config.Paritybit = U0_config.Paritybit - 1;
      temp_uart0 |= 0x08;
    }
  temp_uart0 |= U0_config.Paritybit<<4;
  
  
  U0LCR = temp_uart0; 	// Data Length, Stop bit, Parity bit  setup 
}

/****************************************************************************
* Name: IRQ_Timer1()
* Function: Timer1 interrupt function, inverse LED4(P0.25)
* Input(s): none
* Returns : none
****************************************************************************/
void __irq  IRQ_Timer1(void)
{  
  
  LED_Counter++;

  if( LED_Counter == 50)		// 10ms*50=0.5s led inverse
    {
      if( LED_Flag == 0) LED_Flag = 1;
      else LED_Flag = 0;
  
      LED_Counter = 0;
    }
             
   T1IR = 0x01;	    			// CLR Timer1 interrupt flag
   VICVectAddr = 0x00;			// Inform VIC interrupt is end
}


/****************************************************************************
* Name: IRQ_UART0()
* Function: UART0 interrupt
* Input(s): none
* Returns : none
****************************************************************************/
void   __irq IRQ_UART0(void)
{  
  UInt32 Temp_UART0_IRQ;
  
  if( (U0IIR & 0x0F) == 0x04 )
    {
      Temp_UART0_IRQ = U0RBR - 48;
	  switch( Temp_UART0_IRQ )
	  {
        case 1:
		  	System_Status = 1;   // indicate CAN simple communication
		    break;
		case 2:
		    System_Status = 2;   // indicate CAN communication with ACC
			break;
		case 41:                 // receive "Y"
		case 73:				 // receive "y"
		    if( System_Status == 3) System_Status = 5;
			break;
		case 30:				 // receive "N"
		case 62:				 // receive "n"
			if( System_Status == 3 ) System_Status = 7;
			break;
	  	case 49:		         // when receive "a", reverse LED2
            Temp_UART0_IRQ = IO1SET;
            if( (Temp_UART0_IRQ & LED1CON) == 0 )
              {  
                IO1SET = LED1CON;
              }
            else
              {  
                IO1CLR = LED1CON;
              }
		  	break;
	  	case 50:		// when receive "b", reverse LED1	  	
		  	Temp_UART0_IRQ = IO1SET;
            if( (Temp_UART0_IRQ & LED2CON) == 0 )
              {  
                IO1SET = LED2CON;
              }
            else
              {  
                IO1CLR = LED2CON;
              }              
		  	break;
 		default:
			break;
      }      
    }
  
  
  Temp_UART0_IRQ = U0RBR;		//clear UART0-Rx interrupt flag, when U0IIR != 04,
  VICVectAddr = 0x00;
}


/****************************************************************************
* Name: IRQ_CAN2Rx()
* Function: IRQ_CAN2Rx interrupt
* Input(s): none
* Returns : none
****************************************************************************/
void   __irq IRQ_CAN2Rx(void)
{
  UInt32  Temp_CAN2Rx_IRQ=0;
  UInt8   i=0;
  plpc2000CANdriver_RXObj_t pCAN2_Rcv_Data_IRQ;


  i = 0;
  CAN2Rx_Return_Message_Flag[i] = 1;  //indicate one Rx interrupt happened
  i++;

  Temp_CAN2Rx_IRQ = CAN2ICR;

  if( (Temp_CAN2Rx_IRQ & 0x00000001) == 0x00000001 )
    {
	  // put Rcv data to Rcv-Buffer, must assign value to pointer before used
	  pCAN2_Rcv_Data_IRQ = CAN_Rcv_Data + CAN_Rcv_Data_Counter;
      Return_Value = lpc2000CANdriver_ReceiveMessageCh2 (pCAN2_Rcv_Data_IRQ);
	  CAN_Rcv_Data_Counter++;
	  
	  CAN_Rcv_Status = 1;  	   // communication successful
	}

  if( ((Temp_CAN2Rx_IRQ>>2) & 0x00000001) == 0x00000001 )
    {
	  CAN2Rx_Return_Message_Flag[i] = 1;
	  //Add code for "Error Warning Interrupt-EI"	  
	}
	  i++;

  if( ((Temp_CAN2Rx_IRQ>>3) & 0x00000001) == 0x00000001 )
    {
	  CAN2Rx_Return_Message_Flag[i] = 1;
	  //Add code for "Data Overrun Interrupt-DOI"	  
	}
  i++;

  if( ((Temp_CAN2Rx_IRQ>>4) & 0x00000001) == 0x00000001 )
    {
	  CAN2Rx_Return_Message_Flag[i] = 1;
	  //Add code for "Wake-Up Interrupt-WUI"	  
	}
  i++;

  if( ((Temp_CAN2Rx_IRQ>>5) & 0x00000001) == 0x00000001 )
    {
	  CAN2Rx_Return_Message_Flag[i] = 1;
	  //Add code for "Error Passive Interrupt-EPI"	  
	}
  i++;

  if( ((Temp_CAN2Rx_IRQ>>6) & 0x00000001) == 0x00000001 )
    {
	  CAN2Rx_Return_Message_Flag[i] = 1;
	  //Add code for "Arbitration Lost Interrupt-ALI"	  
	}
  i++;

  if( ((Temp_CAN2Rx_IRQ>>7) & 0x00000001) == 0x00000001 )
    {
	  CAN2Rx_Return_Message_Flag[i] = 1;
	  //Add code for "Bus Error Interrupt-BEI"	  
	}
  i++;

  if( ((Temp_CAN2Rx_IRQ>>8) & 0x00000001) == 0x00000001 )
    {

	  //Add code for "ID Ready Interrupt-IDI"	  
	}


  VICVectAddr = 0x00;
  
}
